<?php

function w2dc_getPluginID() {
	return 6463373;
}
function w2dc_getPluginName() {
	return 'Web 2.0 Directory plugin';
}
function w2dc_getEnvatoSlug() {
	return 'web-20-directory-plugin-for-wordpress';
}
function w2dc_getErrorMessage() {
	return esc_html__("check your purchase code and envato access token at the Directory Admin -> Directory settings page. Make sure you have correctly generated access token https://www.salephpscripts.com/wordpress_directory/demo/documentation/update/", "W2DC");
}
function w2dc_getUpdateDocsLink() {
	return 'http://www.salephpscripts.com/wordpress_directory/demo/documentation/update/';
}
function w2dc_getVersionLink() {
	return 'https://www.salephpscripts.com/wordpress_directory/version/';
}

class w2dc_updater {
	private $slug; // plugin slug
	private $plugin_file; // __FILE__ of our plugin
	private $envato_res;
	
	private $plugin_data;

	private $purchase_code;
	private $api_key; // buyer's Personal Token required
	
	public function __construct($plugin_file, $purchase_code, $access_token) {
		add_filter("pre_set_site_transient_update_plugins", array($this, "setTransient"));
		add_filter("plugins_api", array($this, "setPluginInfo"), 10, 3);
		
		add_filter("upgrader_package_options", array($this, "setUpdatePackage"));
		add_filter("upgrader_pre_download", array($this, "updateErrorMessage"), 10, 3);

		$this->update_path = w2dc_getVersionLink();
		$this->plugin_file = $plugin_file;
		$this->slug = plugin_basename($this->plugin_file);
		
		add_action('in_plugin_update_message-' . $this->slug, array($this, 'showUpgradeMessage'), 10, 2);

		$this->purchase_code = $purchase_code;
		$this->api_key = $access_token;
	}
	
	public function getDownload_url($debug = false) {
		if ($this->purchase_code && $this->api_key) {
			$url = "https://api.envato.com/v3/market/buyer/download?purchase_code=" . $this->purchase_code;
			$curl = curl_init($url);
			
			$header = array();
			$header[] = 'Authorization: Bearer ' . $this->api_key;
			$header[] = 'User-Agent: Purchase code verification on ' . get_bloginfo();
			$header[] = 'timeout: 20';
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl, CURLOPT_HTTPHEADER,$header);
			curl_setopt($curl, CURLOPT_REFERER, $_SERVER["HTTP_HOST"]);
			curl_setopt($curl, CURLOPT_HEADER, 0);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($curl, CURLOPT_ENCODING, 'UTF-8');
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
			
			$this->envato_res = curl_exec($curl);
			curl_close($curl);
			$this->envato_res = json_decode($this->envato_res);
			
			if ($debug) {
				var_dump($this->envato_res);
			}
			
			if (isset($this->envato_res->wordpress_plugin) && strpos($this->envato_res->wordpress_plugin, w2dc_getEnvatoSlug()) !== false) {
				return $this->envato_res->wordpress_plugin;
			} else {
				return $this->envato_res->error;
			}
		}
	}
	
	function updateErrorMessage($reply, $package, $upgrader) {
		if (!empty($package)) {
			return $reply;
		}
		
		if (isset($this->envato_res->error)) {
			$error_message = esc_html__("response from Codecanyon - ", "W2DC") . $this->envato_res->error . (!empty($this->envato_res->description) ? ' (' .  $this->envato_res->description . ')' : '');
		} else {
			$error_message = w2dc_getErrorMessage();
		}
		
		return new WP_Error('no_package', $error_message);
	}
	
	public function getRemote_version() {
		$request = wp_remote_get($this->update_path);
		if (!is_wp_error($request) || wp_remote_retrieve_response_code($request) === 200) {
			return $request['body'];
		}
		
		return false;
	}
	
	public function setUpdatePackage($options) {
		$package = $options['package'];
		if ($package === $this->slug) {
			$options['package'] = $this->getDownload_url();
		}
		
		return $options;
	}
	
	// Push in plugin version information to get the update notification
	public function setTransient($transient) {
		// If we have checked the plugin data before, don't re-check
		if (empty($transient->checked)) {
			return $transient;
		}

		// Get plugin & version information
		$remote_version = $this->getRemote_version();

		// If a newer version is available, add the update
		if (version_compare(W2DC_VERSION, $remote_version, '<')) {
			$plugin_data = get_plugin_data($this->plugin_file);
			
			$obj = new stdClass();
			$obj->slug = str_replace('.php', '', $this->slug);
			$obj->new_version = $remote_version;
			$obj->package = $this->slug;
			$obj->url = $plugin_data["PluginURI"];
			$obj->name = w2dc_getPluginName();
			$transient->response[$this->slug] = $obj;
		}
		
		return $transient;
	}
	
	public function showUpgradeMessage($plugin_data, $response) {
		if (empty($response->package)) {
			echo sprintf(__('Correct Envato access token required. You have to download the latest version from <a href="%s" target="_blank">Codecanyon</a> and follow <a href="%s" target="_blank">update instructions</a>.', 'W2DC'), 'https://codecanyon.net/downloads', w2dc_getUpdateDocsLink());
		}
	}
	
	// Push in plugin version information to display in the details lightbox
	public function setPluginInfo($false, $action, $response) {
		if (empty($response->slug) || $response->slug != str_replace('.php', '', $this->slug)) {
			return $false;
		}
		
		if ($action == 'plugin_information') {
			$remote_version = $this->getRemote_version();

			$plugin_data = get_plugin_data($this->plugin_file);
			
			if ($envatoRes = w2dc_get_plugin_info($this->purchase_code)) {
				$response = new stdClass();
				$response->last_updated = $envatoRes->item->updated_at;
				$response->slug = $this->slug;
				$response->name  = $this->pluginData["Name"];
				$response->plugin_name  = $plugin_data["Name"];
				$response->version = $remote_version;
				$response->author = $plugin_data["AuthorName"];
				$response->homepage = $plugin_data["PluginURI"];
	
				if (isset($envatoRes->item->description)) {
					$response->sections = array(
							'description' => $envatoRes->item->description,
					);
				}
				return $response;
			}
		}
	}
}
																																																																																				
function w2dc_getAccessToken() { return 'R0qSjwSjti1fvlnVB7Kt1rNKgz2cdAYE'; } add_action('vp_w2dc_option_before_ajax_save', 'w2dc_verify_license_on_setting', 1); function w2dc_verify_license_on_setting($opts) { global $w2dc_instance, $w2dc_license_verify_error; $q = "hexdec"; if (!get_option("w2dc_v{$q("0x14")}Qd10fG041L01")) { if (!empty($opts['w2dc_purchase_code'])) { $w2dc_purchase_code = trim($opts['w2dc_purchase_code']); update_option('w2dc_purchase_code', $w2dc_purchase_code); update_option('vpt_option', array( 'w2dc_purchase_code' => $w2dc_purchase_code ) ); if (w2dc_verify_license($w2dc_purchase_code)) { update_option("w2dc_v{$q("0x14")}Qd10fG041L01", 1); if (ob_get_length()) ob_clean(); header('Content-type: application/json'); echo json_encode(array( 'status' => true, 'message' => 'License verification passed successfully!' )); die(); } } remove_action('vp_w2dc_option_after_ajax_save', array($w2dc_instance->settings_manager, 'save_option'), 10); if (ob_get_length()) ob_clean(); header('Content-type: application/json'); echo json_encode(array( 'status' => false, 'message' => 'License verification did not pass!<br />' . $w2dc_license_verify_error )); die(); } } function w2dc_get_plugin_info($purchase_code) { if ($purchase_code) { $url = "https://api.envato.com/v3/market/author/sale?code=".$purchase_code; $curl = curl_init($url); $header = array(); $header[] = 'Authorization: Bearer '.w2dc_getAccessToken(); $header[] = 'User-Agent: Purchase code verification on ' . get_bloginfo(); $header[] = 'timeout: 20'; curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1); curl_setopt($curl, CURLOPT_HTTPHEADER,$header); curl_setopt($curl, CURLOPT_REFERER, $_SERVER["HTTP_HOST"]); curl_setopt($curl, CURLOPT_HEADER, 0); curl_setopt($curl, CURLOPT_ENCODING, 'UTF-8'); curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0); if (($envatoRes = curl_exec($curl)) !== false) { curl_close($curl); return json_decode($envatoRes); } else { global $w2dc_license_verify_error; $w2dc_license_verify_error = "cURL error: " . curl_error($curl); curl_close($curl); } } } function w2dc_verify_license($purchase_code) { $envatoRes = w2dc_get_plugin_info($purchase_code); if (isset($envatoRes->item->id) && $envatoRes->item->id == w2dc_getPluginID()) { return true; } elseif (isset($envatoRes->error)) { global $w2dc_license_verify_error; error_log($envatoRes->error . ' ' . $envatoRes->description); $w2dc_license_verify_error = "Envato: " . $envatoRes->error . ' ' . $envatoRes->description; } elseif (isset($envatoRes->message)) { global $w2dc_license_verify_error; $w2dc_license_verify_error = "Envato: " . $envatoRes->message; } elseif (isset($envatoRes->Message)) { global $w2dc_license_verify_error; $w2dc_license_verify_error = "Envato: " . $envatoRes->Message; } } function w2dc_directories_manager_init($directories_manager) { $xxx = 'directories'; $yyy = 'manager'; add_action('admin_menu', array(${$xxx . '_' . $yyy}, 'menu')); } add_filter('w2dc_build_settings', 'w2dc_verify_license_settings', 100); function w2dc_verify_license_settings($options) { $options['template']['menus']['general']['controls'] = array_merge( array('license' => array( 'type' => 'section', 'title' => __('License information', 'W2DC'), 'fields' => array( array( 'type' => 'textbox', 'name' => 'w2dc_purchase_code', 'label' => __('Purchase code*', 'W2DC'), 'description' => sprintf(__('Use purchase code from your codecanyon <a href="%s" target="_blank">downloads page</a>', 'W2DC'), 'https://codecanyon.net/downloads'), 'default' => get_option('w2dc_purchase_code'), ), array( 'type' => 'textbox', 'name' => 'w2dc_access_token', 'label' => __('Access token', 'W2DC'), 'description' => sprintf(__('Required for automatic updates. Generate an Envato API Personal Token by clicking this <a href="%s" target="_blank">link</a>', 'W2DC'), 'https://build.envato.com/create-token/?purchase:download=t&purchase:verify=t&purchase:list=t'), 'default' => get_option('w2dc_access_token'), ), ), )), $options['template']['menus']['general']['controls'] ); return $options; } add_action('w2dc_settings_panel_top', 'w2dc_settings_panel_top'); function w2dc_settings_panel_top() { $q = "hexdec"; if (!get_option("w2dc_v{$q("0x14")}Qd10fG041L01")) { echo '<div class="error">'; echo '<p>' . sprintf('Your installation of %s was not verified. Any changes in the settings below will not be saved. To verify license information take purchase code from your codecanyon <a href="%s" target="_blank">downloads page</a>.', w2dc_getPluginName(), 'https://codecanyon.net/downloads') . '</p>'; echo '</div>'; } } 

?>